@@ -49,6 +49,7 @@ class PhonopyQHA:
         free_energy=None,
         cv=None,
         entropy=None,
+        pressure=None,
         eos="vinet",
         t_max=None,
         energy_plot_factor=None,
@@ -76,6 +77,8 @@ class PhonopyQHA:
             Temperatures ascending order (T) in K.
             dtype='double'
             shape=(temperatures,)
+        pressure: float,
+            Pressure in GPa that is added to energy as PV term.
         free_energy: array_like
             Phonon Helmholtz free energy (F_ph) in kJ/mol.
             dtype='double'
@@ -88,7 +91,9 @@ class PhonopyQHA:
             Phonon entropy at constant volume (S_ph) in J/K/mol.
             dtype='double'
             shape=(temperatuers, volumes)
-        eos: str
+        pressure: float, optional
+            Pressure in GPa that is added to energy as PV term.
+        eos: str, optional
             Equation of state used for fitting F vs V.
             'vinet', 'murnaghan' or 'birch_murnaghan'.
         t_max: float
@@ -102,7 +107,9 @@ class PhonopyQHA:
             Show log or not.
 
         """
-        self._bulk_modulus = BulkModulus(volumes, electronic_energies, eos=eos)
+        self._bulk_modulus = BulkModulus(
+            volumes, electronic_energies, pressure=pressure, eos=eos
+        )
 
         if temperatures is not None:
             self._qha = QHA(
@@ -112,6 +119,7 @@ class PhonopyQHA:
                 cv,
                 entropy,
                 free_energy,
+                pressure=pressure,
                 eos=eos,
                 t_max=t_max,
                 energy_plot_factor=energy_plot_factor,
@@ -280,9 +288,9 @@ class PhonopyQHA:
         """Write bulk modulus vs temperature in file."""
         self._qha.write_bulk_modulus_temperature(filename=filename)
 
-    def plot_bulk_modulus(self):
+    def plot_bulk_modulus(self, thin_number=10):
         """Return pyplot of bulk modulus fitting curve."""
-        return self._bulk_modulus.plot()
+        return self._bulk_modulus.plot(thin_number=thin_number)
 
     def plot_qha(self, thin_number=10, volume_temp_exp=None):
         """Return pyplot of QHA fitting curves at temperatures."""
