@@ -33,6 +33,7 @@
 # ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 # POSSIBILITY OF SUCH DAMAGE.
 
+import pathlib
 import sys
 from io import StringIO
 
@@ -45,10 +46,10 @@ except ImportError:
     from yaml import Loader
 
 from phonopy.cui.settings import fracval
-from phonopy.harmonic.force_constants import similarity_transformation
 from phonopy.structure.atoms import PhonopyAtoms
 from phonopy.structure.dataset import get_displacements_and_forces
 from phonopy.structure.symmetry import Symmetry, elaborate_borns_and_epsilon
+from phonopy.utils import similarity_transformation
 
 
 #
@@ -475,11 +476,13 @@ def check_force_constants_indices(shape, indices, p2s_map, filename):
     """Check consistency of force constants data type."""
     if shape[0] != shape[1] and p2s_map is not None:
         if len(p2s_map) != len(indices) or (p2s_map != indices).any():
-            text = (
-                "%s file is inconsistent with the calculation setting. "
-                "PRIMITIVE_AXIS may not be set correctly."
-            ) % filename
-            raise RuntimeError(text)
+            lines = [
+                f"{filename} file is inconsistent with the calculation setting. "
+                "PRIMITIVE_AXIS may not be set correctly.",
+                "p2s_map in primitive != p2s_map in file",
+                f"{p2s_map} != {indices}",
+            ]
+            raise RuntimeError("\n".join(lines))
 
 
 def parse_disp_yaml(filename="disp.yaml", return_cell=False):
@@ -919,3 +922,39 @@ def _parse_QHA_data(filename):
             else:
                 data.append([float(x) for x in line.split()])
         return np.array(data)
+
+
+def get_io_module_to_decompress(filename):
+    """Return io-module to decompress file.
+
+    Filename extensions of lzma, xz, gzip, bz2 are supported.
+
+    It is supported to use it like `returned_module.open(filename)`.
+
+    """
+    ext = pathlib.Path(filename).suffix
+    if ext == ".xz" or ext == ".lzma":
+        import lzma
+
+        return lzma
+    elif ext == ".gz":
+        import gzip
+
+        return gzip
+    elif ext == ".bz2":
+        import bz2
+
+        return bz2
+    else:
+        import io
+
+        return io
+
+
+def get_supported_file_extensions_for_compression():
+    """Return file extensions for compression.
+
+    This function must be coupled with `get_io_module_to_decompress`.
+
+    """
+    return "", ".xz", ".lzma", ".gz", ".bz2"
