@@ -33,33 +33,34 @@
 # ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 # POSSIBILITY OF SUCH DAMAGE.
 
+from __future__ import annotations
+
+from typing import TYPE_CHECKING, Optional
+
 import numpy as np
 
+if TYPE_CHECKING:
+    from phonopy import Phonopy
+
 from phonopy.structure.atoms import PhonopyAtoms
 from phonopy.structure.cells import (
+    Primitive,
     compute_permutation_for_rotation,
     get_smallest_vectors,
+    isclose,
 )
+from phonopy.structure.symmetry import Symmetry
+from phonopy.utils import similarity_transformation
 
 
-def get_force_constants(
-    set_of_forces, symmetry, supercell, atom_list=None, decimals=None
+def get_fc2(
+    supercell: PhonopyAtoms,
+    symmetry: Symmetry,
+    dataset,
+    atom_list=None,
+    primitive: Optional[Primitive] = None,
+    decimals=None,
 ):
-    """Calculate force constants from disp-force dataset."""
-    first_atoms = [
-        {
-            "number": x.get_atom_number(),
-            "displacement": x.get_displacement(),
-            "forces": x.get_forces(),
-        }
-        for x in set_of_forces
-    ]
-    dataset = {"natom": supercell.get_number_of_atoms(), "first_atoms": first_atoms}
-    force_constants = get_fc2(supercell, symmetry, dataset, atom_list=atom_list)
-    return force_constants
-
-
-def get_fc2(supercell, symmetry, dataset, atom_list=None, decimals=None):
     """Force constants are computed.
 
     Force constants, Phi, are calculated from sets for forces, F, and
@@ -92,18 +93,33 @@ def get_fc2(supercell, symmetry, dataset, atom_list=None, decimals=None):
     atom_list_done = _get_force_constants_disps(
         force_constants, supercell, dataset, symmetry, atom_list=atom_list
     )
-
     rotations = symmetry.symmetry_operations["rotations"]
     lattice = np.array(supercell.cell.T, dtype="double", order="C")
     permutations = symmetry.atomic_permutations
-    distribute_force_constants(
-        force_constants,
-        atom_list_done,
-        lattice,
-        rotations,
-        permutations,
-        atom_list=atom_list,
-    )
+
+    if atom_list is None and primitive is not None:
+        # Distribute to atoms in primitive cell, then distribute to all.
+        distribute_force_constants(
+            force_constants,
+            atom_list_done,
+            lattice,
+            rotations,
+            permutations,
+            atom_list=primitive.p2s_map,
+            fc_indices_of_atom_list=primitive.p2s_map,
+        )
+        distribute_force_constants_by_translations(
+            force_constants, primitive, supercell
+        )
+    else:
+        distribute_force_constants(
+            force_constants,
+            atom_list_done,
+            lattice,
+            rotations,
+            permutations,
+            atom_list=atom_list,
+        )
 
     if decimals:
         force_constants = force_constants.round(decimals=decimals)
@@ -111,7 +127,7 @@ def get_fc2(supercell, symmetry, dataset, atom_list=None, decimals=None):
     return force_constants
 
 
-def compact_fc_to_full_fc(phonon, compact_fc, log_level=0):
+def compact_fc_to_full_fc(phonon: "Phonopy", compact_fc, log_level=0):
     """Transform compact fc to full fc."""
     fc = np.zeros(
         (compact_fc.shape[1], compact_fc.shape[1], 3, 3), dtype="double", order="C"
@@ -124,6 +140,45 @@ def compact_fc_to_full_fc(phonon, compact_fc, log_level=0):
     return fc
 
 
+def full_fc_to_compact_fc(phonon: "Phonopy", full_fc, log_level=0):
+    """Transform full fc to compact fc."""
+    p2s_map = phonon.primitive.p2s_map
+    fc = np.zeros((len(p2s_map), full_fc.shape[1], 3, 3), dtype="double", order="C")
+    fc[:] = full_fc[p2s_map]
+    if log_level:
+        print("Force constants format was transformed to compact format.")
+
+    return fc
+
+
+def rearrange_force_constants_array(
+    full_fc: np.ndarray, a: PhonopyAtoms, b: PhonopyAtoms
+) -> tuple[np.ndarray, list]:
+    """Rearrange force constants array of cell-a to those of cell-b.
+
+    Parameters
+    ----------
+    full_fc : ndarray
+        Force constants in full array format.
+    a : PhonopyAtoms
+        Cell of input force constants.
+    b : PhonopyAtoms
+        Cell of rearranged force constants.
+
+    Returns
+    -------
+    ndarray
+        Rearranged force constants array.
+
+    """
+    assert full_fc.shape[0] == full_fc.shape[1]
+    indices: list = isclose(a, b, with_arbitrary_order=True, return_order=True)
+    re_fc = np.zeros_like(full_fc)
+    for i, j in enumerate(indices):
+        re_fc[i] = full_fc[j][indices]
+    return re_fc, indices
+
+
 def cutoff_force_constants(
     force_constants, supercell, primitive, cutoff_radius, symprec=1e-5
 ):
@@ -196,7 +251,7 @@ def symmetrize_force_constants(force_constants, level=1):
         set_translational_invariance(force_constants)
 
 
-def symmetrize_compact_force_constants(force_constants, primitive, level=1):
+def symmetrize_compact_force_constants(force_constants, primitive: Primitive, level=1):
     """Symmetry force constants by translational and permutation symmetries.
 
     Parameters
@@ -239,8 +294,17 @@ def distribute_force_constants(
     rotations,  # scaled (fractional)
     permutations,
     atom_list=None,
+    fc_indices_of_atom_list=None,
 ):
-    """Fill force constants elements by symmetry."""
+    """Fill force constants elements by symmetry.
+
+    atom_list : ndarray
+        Indices of target atoms.
+    fc_indices_of_atom_list : ndarray
+        First fc3 indices corresponding to indices of target atoms. Unless
+        specified, `range(len(atom_list))` is used.
+
+    """
     map_atoms, map_syms = _get_sym_mappings_from_permutations(
         permutations, atom_list_done
     )
@@ -253,11 +317,16 @@ def distribute_force_constants(
         targets = np.arange(force_constants.shape[1], dtype="intc")
     else:
         targets = np.array(atom_list, dtype="intc")
+    if fc_indices_of_atom_list is None:
+        _fc_indices_of_atom_list = np.arange(len(targets), dtype="intc")
+    else:
+        _fc_indices_of_atom_list = np.array(fc_indices_of_atom_list, dtype="intc")
     import phonopy._phonopy as phonoc
 
     phonoc.distribute_fc2(
         force_constants,
         targets,
+        _fc_indices_of_atom_list,
         rots_cartesian,
         permutations,
         np.array(map_atoms, dtype="intc"),
@@ -265,7 +334,9 @@ def distribute_force_constants(
     )
 
 
-def distribute_force_constants_by_translations(fc, primitive, supercell):
+def distribute_force_constants_by_translations(
+    fc, primitive: Primitive, supercell: PhonopyAtoms
+):
     """Distribute compact fc data to full fc by pure translations.
 
     For example, the input fc has to be prepared in the following way
@@ -276,16 +347,12 @@ def distribute_force_constants_by_translations(fc, primitive, supercell):
     fc[primitive.p2s_map] = compact_fc
 
     """
-    s2p = primitive.s2p_map
     p2s = primitive.p2s_map
-    positions = supercell.scaled_positions
-    lattice = supercell.cell.T
-    diff = positions - positions[p2s[0]]
-    trans = np.array(diff[np.where(s2p == p2s[0])[0]], dtype="double", order="C")
-    rotations = np.array(
-        [np.eye(3, dtype="intc")] * len(trans), dtype="intc", order="C"
-    )
     permutations = primitive.atomic_permutations
+    lattice = supercell.cell.T
+    rotations = np.array(
+        [np.eye(3, dtype="intc")] * permutations.shape[0], dtype="intc", order="C"
+    )
     distribute_force_constants(fc, p2s, lattice, rotations, permutations)
 
 
@@ -469,11 +536,6 @@ def set_permutation_symmetry(force_constants):
             force_constants[i, j] = (force_constants[i, j] + fc_copy[j, i].T) / 2
 
 
-def similarity_transformation(rot, mat):
-    """Similarity transformation by R x M x R^-1."""
-    return np.dot(rot, np.dot(mat, np.linalg.inv(rot)))
-
-
 def show_drift_force_constants(
     force_constants, primitive=None, name="force constants", values_only=False
 ):
@@ -769,7 +831,7 @@ def _get_atom_indices_by_symmetry(lattice, positions, rotations, translations, s
     diff -= np.rint(diff)
     diff = np.dot(diff, lattice.T)
     # m[N, K(1), K(2)]
-    m = np.sqrt(np.sum(diff ** 2, axis=3)) < symprec
+    m = np.sqrt(np.sum(diff**2, axis=3)) < symprec
     # index_array[K(1), K(2)]
     index_array = np.tile(np.arange(K, dtype="intc"), (K, 1))
     # Understanding numpy boolean array indexing (extract True elements)
