@@ -33,15 +33,21 @@
 # ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 # POSSIBILITY OF SUCH DAMAGE.
 
+import os
 import sys
 import warnings
 import xml.etree.cElementTree as etree
 import xml.parsers.expat
 from io import StringIO
+from typing import Optional
 
 import numpy as np
 
-from phonopy.file_IO import write_FORCE_CONSTANTS, write_force_constants_to_hdf5
+from phonopy.file_IO import (
+    get_io_module_to_decompress,
+    write_FORCE_CONSTANTS,
+    write_force_constants_to_hdf5,
+)
 from phonopy.structure.atoms import PhonopyAtoms, atom_data, symbol_map
 from phonopy.structure.symmetry import elaborate_borns_and_epsilon
 from phonopy.units import VaspToTHz
@@ -116,7 +122,8 @@ def parse_set_of_forces(num_atoms, forces_filenames, use_expat=True, verbose=Tru
     force_files = forces_filenames
 
     for filename in force_files:
-        with open(filename, "rb") as fp:
+        myio = get_io_module_to_decompress(filename)
+        with myio.open(filename, "rb") as fp:
             if verbose:
                 sys.stdout.write("%d " % (count + 1))
             vasprun = Vasprun(fp, use_expat=use_expat)
@@ -189,8 +196,11 @@ def parse_force_constants(filename):
         force constants and chemical elements
 
     """
-    vasprun = Vasprun(open(filename, "rb"))
-    return vasprun.read_force_constants()
+    myio = get_io_module_to_decompress(filename)
+    with myio.open(filename, "rb") as f:
+        vasprun = Vasprun(f)
+        fc = vasprun.read_force_constants()
+    return fc
 
 
 #
@@ -307,20 +317,7 @@ def write_vasp(filename, cell, direct=True):
 
 def get_vasp_structure_lines(cell, direct=True, is_vasp5=True):
     """Generate POSCAR text lines as a list from PhonopyAtoms instance."""
-    (num_atoms, symbols, scaled_positions, sort_list) = sort_positions_by_symbols(
-        cell.symbols, cell.scaled_positions
-    )
-    lines = []
-    if is_vasp5:
-        lines.append("generated by phonopy")
-    else:
-        lines.append(" ".join(["%s" % s for s in symbols]))
-    lines.append("   1.0")
-    for a in cell.cell:
-        lines.append("  %21.16f %21.16f %21.16f" % tuple(a))
-    if is_vasp5:
-        lines.append(" ".join(["%s" % s for s in symbols]))
-    lines.append(" ".join(["%4d" % n for n in num_atoms]))
+    lines, scaled_positions = _get_vasp_structure_header_lines(cell, is_vasp5=is_vasp5)
     lines.append("Direct")
     lines += _get_scaled_positions_lines(scaled_positions)
 
@@ -343,6 +340,24 @@ def write_supercells_with_displacements(
         write_vasp(filename, cell, direct=True)
 
 
+def _get_vasp_structure_header_lines(cell: PhonopyAtoms, is_vasp5=True):
+    (num_atoms, symbols, scaled_positions, sort_list) = sort_positions_by_symbols(
+        cell.symbols, cell.scaled_positions
+    )
+    lines = []
+    if is_vasp5:
+        lines.append("generated by phonopy")
+    else:
+        lines.append(" ".join(["%s" % s for s in symbols]))
+    lines.append("   1.0")
+    for a in cell.cell:
+        lines.append("  %21.16f %21.16f %21.16f" % tuple(a))
+    if is_vasp5:
+        lines.append(" ".join(["%s" % s for s in symbols]))
+    lines.append(" ".join(["%4d" % n for n in num_atoms]))
+    return lines, scaled_positions
+
+
 def _get_scaled_positions_lines(scaled_positions):
     # map into 0 <= x < 1.
     # (the purpose of the second '% 1' is to handle a surprising
@@ -398,7 +413,8 @@ def get_born_vasprunxml(
     See elaborate_borns_and_epsilon.
 
     """
-    with open(filename, "rb") as f:
+    myio = get_io_module_to_decompress(filename)
+    with myio.open(filename, "rb") as f:
         vasprun = VasprunxmlExpat(f)
         try:
             vasprun.parse()
@@ -462,7 +478,8 @@ def get_born_OUTCAR(
 
 
 def _read_born_and_epsilon_from_OUTCAR(filename):
-    with open(filename) as outcar:
+    myio = get_io_module_to_decompress(filename)
+    with myio.open(filename, mode="rt") as outcar:
         borns = []
         epsilon = []
 
@@ -576,7 +593,7 @@ class Vasprun:
                 for element_i in element.findall("./i"):
                     if element_i.attrib["name"] == "version":
                         version_str = element_i.text.strip()
-                        version_nums = [int(v) for v in version_str.split(".")]
+                        version_nums = version_str.split("-")[0].split(".")
 
             if num_atom == 0:
                 atomtypes = self._get_atomtypes(element)
@@ -615,8 +632,8 @@ class Vasprun:
 
             # Recover the unit of eV/Angstrom^2 for VASP-6.
             if version_nums is not None and len(version_nums) > 1:
-                if version_nums[0] == 6 and version_nums[1] > 1:
-                    force_constants /= VaspToTHz ** 2
+                if int(version_nums[0]) == 6 and int(version_nums[1]) > 1:
+                    force_constants /= VaspToTHz**2
 
             return force_constants, elements
 
@@ -712,6 +729,8 @@ class VasprunxmlExpat:
         self._is_volume = False
         self._is_energy = False
         self._is_k_weights = False
+        self._is_kpoints = False
+        self._is_kpointlist = False
         self._is_eigenvalues = False
         self._is_epsilon = False
         self._is_born = False
@@ -719,6 +738,8 @@ class VasprunxmlExpat:
         self._is_generation = False
         self._is_divisions = False
         self._is_NELECT = False
+        self._is_NGXYZ = [False, False, False]
+        self._is_NGXYZF = [False, False, False]
 
         self._is_v = False
         self._is_i = False
@@ -727,6 +748,8 @@ class VasprunxmlExpat:
         self._is_set = False
         self._is_r = False
         self._is_field = False
+        self._is_separator = False
+        self._is_grids = False
 
         self._is_scstep = False
         self._is_structure = False
@@ -750,6 +773,7 @@ class VasprunxmlExpat:
         self._epsilon = None
         self._born_atom = None
         self._k_weights = None
+        self._kpointlist = None
         self._k_mesh = None
         self._eigenvalues = None
         self._eig_state = [0, 0]
@@ -758,6 +782,11 @@ class VasprunxmlExpat:
         self._field_val = None
         self._pseudopotentials = []
         self._ps_atom = None
+        self._fft_grid = [0, 0, 0]
+        self._fft_fine_grid = [0, 0, 0]
+        self._efermi = None
+        self._symbols = None
+        self._NELECT = None
 
         self._p = xml.parsers.expat.ParserCreate()
         self._p.buffer_text = True
@@ -765,10 +794,6 @@ class VasprunxmlExpat:
         self._p.EndElementHandler = self._end_element
         self._p.CharacterDataHandler = self._char_data
 
-        self.efermi = None
-        self.symbols = None
-        self.NELECT = None
-
         self._cbuf = None
 
     def parse(self):
@@ -817,6 +842,11 @@ class VasprunxmlExpat:
         )
         return self.epsilon
 
+    @property
+    def efermi(self):
+        """Return Fermi energy."""
+        return self._efermi
+
     def get_efermi(self):
         """Return efermi."""
         warnings.warn(
@@ -824,7 +854,7 @@ class VasprunxmlExpat:
             "Use VasprunxmlExpat.efermi attribute.",
             DeprecationWarning,
         )
-        return self.efermi
+        return self._efermi
 
     @property
     def born(self):
@@ -872,6 +902,11 @@ class VasprunxmlExpat:
         """Return all cell volumes of structure optimization steps."""
         return np.array(self._all_volumes, dtype="double")
 
+    @property
+    def symbols(self):
+        """Return atomic symbols."""
+        return self._symbols
+
     def get_symbols(self):
         """Return atomic symbols."""
         warnings.warn(
@@ -879,7 +914,7 @@ class VasprunxmlExpat:
             "Use VasprunxmlExpat.symbols attribute.",
             DeprecationWarning,
         )
-        return self.symbols
+        return self._symbols
 
     @property
     def energies(self):
@@ -909,6 +944,11 @@ class VasprunxmlExpat:
         """Return k_mesh."""
         return np.array(self._k_mesh, dtype="intc")
 
+    @property
+    def kpointlist(self):
+        """Return kpoint list."""
+        return np.array(self._kpointlist, dtype="double")
+
     @property
     def k_weights(self):
         """Return k_weights.
@@ -1012,11 +1052,26 @@ class VasprunxmlExpat:
     def cell(self):
         """Return cell in PhonopyAtoms."""
         return PhonopyAtoms(
-            symbols=self.symbols,
+            symbols=self._symbols,
             scaled_positions=self.points[-1],
             cell=self.lattice[-1],
         )
 
+    @property
+    def fft_grid(self):
+        """Return FFT gird [NGX, NGY, NGZ]."""
+        return self._fft_grid
+
+    @property
+    def fft_fine_grid(self):
+        """Return fine FFT gird [NGXF, NGYF, NGZF]."""
+        return self._fft_fine_grid
+
+    @property
+    def NELECT(self):
+        """Return number of electrons, NELECT."""
+        return self._NELECT
+
     def _start_element(self, name, attrs):
         # Used not to collect energies in <scstep>
         if name == "scstep":
@@ -1038,6 +1093,7 @@ class VasprunxmlExpat:
             or self._is_basis
             or self._is_volume
             or self._is_k_weights
+            or self._is_kpointlist
             or self._is_generation
         ):
             if name == "v":
@@ -1061,6 +1117,10 @@ class VasprunxmlExpat:
                     self._is_k_weights = True
                     self._k_weights = []
 
+                if attrs["name"] == "kpointlist":
+                    self._is_kpointlist = True
+                    self._kpointlist = []
+
                 if attrs["name"] == "epsilon" or attrs["name"] == "epsilon_scf":
                     self._is_epsilon = True
                     self._epsilon = []
@@ -1074,6 +1134,9 @@ class VasprunxmlExpat:
                         self._is_basis = True
                         self._lattice = []
 
+        if name == "kpoints":
+            self._is_kpoints = True
+
         if name == "field":
             if "type" in attrs:
                 self._cbuf = ""
@@ -1097,6 +1160,24 @@ class VasprunxmlExpat:
                 if not self._is_structure and attrs["name"] == "volume":
                     self._is_i = True
                     self._is_volume = True
+                if attrs["name"] == "NGX":
+                    self._is_i = True
+                    self._is_NGXYZ[0] = True
+                if attrs["name"] == "NGY":
+                    self._is_i = True
+                    self._is_NGXYZ[1] = True
+                if attrs["name"] == "NGZ":
+                    self._is_i = True
+                    self._is_NGXYZ[2] = True
+                if attrs["name"] == "NGXF":
+                    self._is_i = True
+                    self._is_NGXYZF[0] = True
+                if attrs["name"] == "NGYF":
+                    self._is_i = True
+                    self._is_NGXYZF[1] = True
+                if attrs["name"] == "NGZF":
+                    self._is_i = True
+                    self._is_NGXYZF[2] = True
 
         if self._is_energy and name == "i":
             self._cbuf = ""
@@ -1131,7 +1212,7 @@ class VasprunxmlExpat:
             if "name" in attrs.keys():
                 if attrs["name"] == "atoms":
                     self._is_symbols = True
-                    self.symbols = []
+                    self._symbols = []
 
                 if attrs["name"] == "born_charges":
                     self._is_born = True
@@ -1182,6 +1263,11 @@ class VasprunxmlExpat:
                 self._is_eigenvalues = True
                 self._eigenvalues = []
 
+        if name == "separator":
+            self._is_separator = True
+            if attrs["name"] == "grids":
+                self._is_grids = True
+
     def _end_element(self, name):
         if name == "scstep":
             self._is_scstep = False
@@ -1201,6 +1287,9 @@ class VasprunxmlExpat:
             if self._is_k_weights:
                 self._is_k_weights = False
 
+            if self._is_kpointlist:
+                self._is_kpointlist = False
+
             if self._is_positions:
                 self._is_positions = False
                 self._all_points.append(self._points)
@@ -1216,6 +1305,10 @@ class VasprunxmlExpat:
             if self._is_generation:
                 self._is_generation = False
 
+        if name == "kpoints":
+            if self._is_kpoints:
+                self._is_kpoints = False
+
         if name == "array":
             if self._is_symbols:
                 self._is_symbols = False
@@ -1236,17 +1329,11 @@ class VasprunxmlExpat:
         if name == "i":
             self._run_i()
             self._is_i = False
-            if self._is_efermi:
-                self._is_efermi = False
-            if self._is_NELECT:
-                self._is_NELECT = False
-            if self._is_volume:
-                self._is_volume = False
 
         if name == "rc":
             self._is_rc = False
             if self._is_symbols:
-                self.symbols.pop(-1)
+                self._symbols.pop(-1)
 
         if name == "c":
             self._run_c()
@@ -1285,6 +1372,11 @@ class VasprunxmlExpat:
             if name == "c":
                 self._is_c = False
 
+        if name == "separator":
+            if self._is_grids:
+                self._is_grids = False
+            self._is_separator = False
+
     def _run_v(self):
         if self._is_v:
             if self._is_forces:
@@ -1297,13 +1389,18 @@ class VasprunxmlExpat:
                 self._points.append([self._to_float(x) for x in self._cbuf.split()])
             if self._is_basis:
                 self._lattice.append([self._to_float(x) for x in self._cbuf.split()])
-            if self._is_k_weights:
-                self._k_weights.append(self._to_float(self._cbuf))
             if self._is_born:
                 self._born_atom.append([self._to_float(x) for x in self._cbuf.split()])
-            if self._is_generation:
-                if self._is_divisions:
-                    self._k_mesh = [self._to_int(x) for x in self._cbuf.split()]
+            if self._is_kpoints:
+                if self._is_k_weights:
+                    self._k_weights.append(self._to_float(self._cbuf))
+                if self._is_kpointlist:
+                    self._kpointlist.append(
+                        [self._to_float(x) for x in self._cbuf.split()]
+                    )
+                if self._is_generation:
+                    if self._is_divisions:
+                        self._k_mesh = [self._to_int(x) for x in self._cbuf.split()]
             self._cbuf = None
 
     def _run_i(self):
@@ -1311,17 +1408,29 @@ class VasprunxmlExpat:
             if self._is_energy:
                 self._energies.append(self._to_float(self._cbuf.strip()))
             if self._is_efermi:
-                self.efermi = self._to_float(self._cbuf.strip())
+                self._efermi = self._to_float(self._cbuf.strip())
+                self._is_efermi = False
             if self._is_NELECT:
-                self.NELECT = self._to_float(self._cbuf.strip())
+                self._NELECT = self._to_float(self._cbuf.strip())
+                self._is_NELECT = False
             if self._is_volume:
                 self._all_volumes.append(self._to_float(self._cbuf.strip()))
+                self._is_volume = False
+            if self._is_grids:
+                for i, b in enumerate(self._is_NGXYZ):
+                    if b:
+                        self._fft_grid[i] = self._to_int(self._cbuf.strip())
+                        self._is_NGXYZ[i] = False
+                for i, b in enumerate(self._is_NGXYZF):
+                    if b:
+                        self._fft_fine_grid[i] = self._to_int(self._cbuf.strip())
+                        self._is_NGXYZF[i] = False
             self._cbuf = None
 
     def _run_c(self):
         if self._is_c:
             if self._is_symbols:
-                self.symbols.append(str(self._cbuf.strip()))
+                self._symbols.append(str(self._cbuf.strip()))
             if self._field_val == "pseudopotential" and self._is_set and self._is_rc:
                 if len(self._ps_atom) == 0:
                     self._ps_atom.append(self._to_int(self._cbuf.strip()))
@@ -1371,12 +1480,36 @@ class VasprunxmlExpat:
             raise
 
 
+def parse_vasprunxml(filename):
+    """Parse vasprun.xml using VasprunxmlExpat."""
+    if not os.path.exists(filename):
+        print("File %s not found." % filename)
+        sys.exit(1)
+
+    myio = get_io_module_to_decompress(filename)
+    with myio.open(filename, "rb") as f:
+        vxml = VasprunxmlExpat(f)
+        try:
+            vxml.parse()
+        except (xml.parsers.expat.ExpatError, ValueError, Exception):
+            print("Warning: Probably xml structure of %s is broken." % filename)
+            print("Opening %s failed." % filename)
+            sys.exit(1)
+        return vxml
+
+
 #
 # XDATCAR
 #
-def read_XDATCAR(filename="XDATCAR"):
+def read_XDATCAR(filename: str = "XDATCAR", fileptr=None):
     """Read XDATCAR.
 
+    filename : str, optional
+        Input filename in `XDATCAR` format. Default is `XDATCAR`. This is used
+        unless fileptr is specified.
+    fileptr : readable file pointer, optional
+        File pointer used to read `XDATCAR`.
+
     Returns
     -------
     tuple of (lattice, positions)
@@ -1390,22 +1523,22 @@ def read_XDATCAR(filename="XDATCAR"):
 
     """
     lattice = None
-    symbols = None
     numbers_of_atoms = None
-    with open(filename) as f:
-        f.readline()
-        scale = float(f.readline())
-        a = [float(x) for x in f.readline().split()[:3]]
-        b = [float(x) for x in f.readline().split()[:3]]
-        c = [float(x) for x in f.readline().split()[:3]]
-        lattice = np.transpose([a, b, c]) * scale
-        symbols = f.readline().split()
-        numbers_of_atoms = np.array(
-            [int(x) for x in f.readline().split()[: len(symbols)]], dtype="intc"
-        )
+
+    if fileptr is None:
+        myio = get_io_module_to_decompress(filename)
+        with myio.open(filename) as f:
+            lattice, numbers_of_atoms = _read_XDATCAR_fileptr(f)
+    else:
+        lattice, numbers_of_atoms = _read_XDATCAR_fileptr(fileptr)
 
     if lattice is not None:
-        data = np.loadtxt(filename, skiprows=7, comments="D", dtype="double")
+        if fileptr is None:
+            _file = filename
+        else:
+            _file = fileptr
+            _file.seek(0)
+        data = np.loadtxt(_file, skiprows=7, comments="D", dtype="double")
         pos = data.reshape((-1, numbers_of_atoms.sum(), 3), order="C")
         lat = np.array(lattice, dtype="double", order="C")
         return lat, pos
@@ -1413,6 +1546,66 @@ def read_XDATCAR(filename="XDATCAR"):
         return None
 
 
+def _read_XDATCAR_fileptr(f):
+    f.readline()
+    scale = float(f.readline())
+    a = [float(x) for x in f.readline().split()[:3]]
+    b = [float(x) for x in f.readline().split()[:3]]
+    c = [float(x) for x in f.readline().split()[:3]]
+    lattice = np.transpose([a, b, c]) * scale
+    symbols = f.readline().split()
+    numbers_of_atoms = np.array(
+        [int(x) for x in f.readline().split()[: len(symbols)]], dtype="intc"
+    )
+    return lattice, numbers_of_atoms
+
+
+def write_XDATCAR(
+    vasprunxml_filename: str = "vasprun.xml",
+    vasprunxml_expat: Optional[VasprunxmlExpat] = None,
+    filename: str = "XDATCAR",
+    fileptr=None,
+    shift=None,
+):
+    """Write XDATCAR from vasprun.xml or VasprunxmlExpat instance.
+
+    vasprunxml_filename : str, optional
+        File name of vasprun.xml. Default is "vasprun.xml". This is used unless
+        `vasprunxml_expat` is specified.
+    vasprunxml_expat : VasprunxmlExpat, optional
+        Instalce of `VasprunxmlExpat`. It is assumed that this instance is
+        already parsed. Default is None.
+    filename : str, optional
+        Output filename in `XDATCAR` format. Default is `XDATCAR`. This is used
+        unless fileptr is specified.
+    fileptr : writable file pointer, optional
+        File pointer used to write `XDATCAR`.
+    shift : array_like, optional
+        All atoms are uniformly translated in reduced coordinates. Default is
+        None. shape=(3,), dtype='double'.
+
+    """
+    if vasprunxml_expat is not None:
+        vxml = vasprunxml_expat
+    else:
+        with open(vasprunxml_filename, "rb") as f:
+            vxml = VasprunxmlExpat(f)
+            vxml.parse()
+    lines, _ = _get_vasp_structure_header_lines(vxml.cell)
+    points = vxml.points
+    if shift is not None:
+        points += shift
+    for i, scaled_positions in enumerate(points):
+        lines.append(f"Direct configuration=    {i}")
+        lines += _get_scaled_positions_lines(scaled_positions)
+
+    if fileptr is None:
+        with open(filename, "w") as w:
+            w.write("\n".join(lines))
+    else:
+        fileptr.write(("\n".join(lines)).encode("utf-8"))
+
+
 #
 # OUTCAR handling (obsolete)
 #
