@@ -37,11 +37,11 @@ import sys
 
 import numpy as np
 
-from phonopy.harmonic.force_constants import similarity_transformation
 from phonopy.interface.vasp import check_forces, get_drift_forces
 from phonopy.structure.atoms import PhonopyAtoms as Atoms
 from phonopy.structure.cells import get_angles, get_cell_parameters
 from phonopy.structure.symmetry import Symmetry
+from phonopy.utils import similarity_transformation
 
 
 def parse_set_of_forces(
@@ -286,7 +286,7 @@ def _transform_axis(alpha, beta, gamma, a, b, c):
 
     az = np.cos(beta) * a
     ay = (np.cos(gamma) - np.cos(beta) * np.cos(alpha)) / np.sin(alpha) * a
-    ax = np.sqrt(a ** 2 - ay ** 2 - az ** 2)
+    ax = np.sqrt(a**2 - ay**2 - az**2)
 
     return [ax, ay, az], [0, by, bz], [0, 0, cz]
 
