@@ -49,43 +49,65 @@ class BulkModulus:
 
     """
 
-    def __init__(self, volumes, energies, eos="vinet"):
+    def __init__(self, volumes, energies, pressure=None, eos="vinet"):
         """Init method.
 
         volumes : array_like
             Unit cell volumes where energies are obtained.
             shape=(volumes, ), dtype='double'.
         energies : array_like
-            Energies obtained at volumes.
-            shape=(volumes, ), dtype='double'.
-        eos : str
+            Energies obtained at volumes and optionally temperatures.
+            shape=(volumes, ), or shape=(temperatures, volumes) dtype='double'.
+        pressure: float, optional
+            Pressure in GPa that is added to energy as PV term.
+        eos : str, optional
             Identifier of equation of states function.
 
         """
-        self._volumes = volumes
-        if np.array(energies).ndim == 1:
-            self._energies = energies
-        else:
-            self._energies = energies[0]
-        self._eos = get_eos(eos)
+        self._volumes = np.array(volumes)
+        self._energies = np.array(energies)
+        self._eos_name = eos
+        self._eos = get_eos(self._eos_name)
 
-        self._energy = None
-        self._bulk_modulus = None
-        self._b_prime = None
+        if pressure is not None:
+            self._energies += self._volumes * pressure / EVAngstromToGPa
 
-        try:
+        if self._energies.ndim == 1:
+            self._energy = None
+            self._bulk_modulus = None
+            self._b_prime = None
+            self._equiv_volume = None
             (
                 self._energy,
                 self._bulk_modulus,
                 self._b_prime,
-                self._volume,
-            ) = fit_to_eos(volumes, self._energies, self._eos)
+                self._equiv_volume,
+            ) = self.fit_to_eos(self._energies)
+        elif self._energies.ndim == 2:
+            self._energy = np.zeros(len(self._energies), dtype="double")
+            self._bulk_modulus = np.zeros_like(self._energy)
+            self._b_prime = np.zeros_like(self._energy)
+            self._equiv_volume = np.zeros_like(self._energy)
+            for i, energies_at_T in enumerate(self._energies):
+                e, b, bp, ev = self.fit_to_eos(energies_at_T)
+                self._energy[i] = e
+                self._bulk_modulus[i] = b
+                self._b_prime[i] = bp
+                self._equiv_volume[i] = ev
+        else:
+            raise TypeError("Array shape of energies is wrong.")
+
+    def fit_to_eos(self, energies):
+        """Fit energy-volume to EOS."""
+        try:
+            e, b, bp, ev = fit_to_eos(self._volumes, energies, self._eos)
         except TypeError:
-            msg = ['Failed to fit to "%s" equation of states.' % eos]
-            if len(volumes) < 4:
+            msg = ['Failed to fit to "%s" equation of states.' % self._eos_name]
+            if len(self._volumes) < 4:
                 msg += ["At least 4 volume points are needed for the fitting."]
             msg += ["Careful choice of volume points is recommended."]
             raise RuntimeError("\n".join(msg))
+        return e, b, bp, ev
 
     @property
     def bulk_modulus(self):
@@ -104,7 +126,7 @@ class BulkModulus:
     @property
     def equilibrium_volume(self):
         """Return volume at equilibrium."""
-        return self._volume
+        return self._equiv_volume
 
     def get_equilibrium_volume(self):
         """Return volume at equilibrium."""
@@ -145,23 +167,30 @@ class BulkModulus:
 
     def get_parameters(self):
         """Return fitted parameters."""
-        return (self._energy, self._bulk_modulus, self._b_prime, self._volume)
+        return (self._energy, self._bulk_modulus, self._b_prime, self._equiv_volume)
 
     def get_eos(self):
         """Return EOS function as a python method."""
         warnings.warn("BulkModulus.get_eos() is deprecated.", DeprecationWarning)
         return self._eos
 
-    def plot(self):
+    def plot(self, thin_number=10):
         """Plot fitted EOS curve."""
         import matplotlib.pyplot as plt
 
-        ep = self.get_parameters()
         vols = self._volumes
         volume_points = np.linspace(min(vols), max(vols), 201)
-        fig, ax = plt.subplots()
-        ax.plot(volume_points, self._eos(volume_points, *ep), "r-")
-        ax.plot(vols, self._energies, "bo", markersize=4)
+        _, ax = plt.subplots()
+        parameters = self.get_parameters()
+        if self._energies.ndim == 1:
+            ax.plot(volume_points, self._eos(volume_points, *parameters), "r-")
+            ax.plot(vols, self._energies, "bo", markersize=4)
+        elif self._energies.ndim == 2:
+            for i, (e_t, b_t, bp_t, ev_t) in enumerate(zip(*parameters)):
+                if i % thin_number == 0:
+                    ep = (e_t, b_t, bp_t, ev_t)
+                    ax.plot(volume_points, self._eos(volume_points, *ep), "-")
+                    ax.plot(vols, self._energies[i], "bo", markersize=4)
         return plt
 
 
@@ -176,6 +205,7 @@ class QHA:
         cv,  # J/K/mol
         entropy,  # J/K/mol
         fe_phonon,  # kJ/mol
+        pressure=None,
         eos="vinet",
         t_max=None,
         energy_plot_factor=None,
@@ -209,7 +239,9 @@ class QHA:
             Phonon Helmholtz free energy (F_ph) in kJ/mol.
             dtype='double'
             shape=(temperatuers, volumes)
-        eos: str
+        pressure: float, optional
+            Pressure in GPa that is added to energy as PV term.
+        eos: str, optional
             Equation of state used for fitting F vs V.
             'vinet', 'murnaghan' or 'birch_murnaghan'.
         t_max: float
@@ -223,7 +255,8 @@ class QHA:
         """
         self._volumes = np.array(volumes)
         self._electronic_energies = np.array(electronic_energies)
-
+        if pressure is not None:
+            self._electronic_energies += self._volumes * pressure / EVAngstromToGPa
         self._all_temperatures = np.array(temperatures)
         self._cv = np.array(cv)
         self._entropy = np.array(entropy)
@@ -298,7 +331,9 @@ class QHA:
         if self._electronic_energies.ndim == 1:
             return self._cp_polyfit[: self._len]
         else:
-            return None
+            raise NotImplementedError(
+                "QHA.heat_capacity_P_polyfit() unavailable with electronic free energy"
+            )
 
     @property
     def gruneisen_temperature(self):
@@ -1078,7 +1113,7 @@ class QHA:
                     msg += ["At least 5 volume points are needed for the fitting."]
                 raise RuntimeError("\n".join(msg))
 
-            cv_p = np.dot(parameters, np.array([x ** 4, x ** 3, x ** 2, x, 1]))
+            cv_p = np.dot(parameters, np.array([x**4, x**3, x**2, x, 1]))
             self._volume_cv_parameters.append(parameters)
 
             try:
@@ -1090,7 +1125,7 @@ class QHA:
                 raise RuntimeError("\n".join(msg))
 
             dsdv_t = np.dot(
-                parameters[:4], np.array([4 * x ** 3, 3 * x ** 2, 2 * x, 1])
+                parameters[:4], np.array([4 * x**3, 3 * x**2, 2 * x, 1])
             )
             self._volume_entropy_parameters.append(parameters)
 
@@ -1131,7 +1166,7 @@ class QHA:
                     msg += ["At least 5 volume points are needed for the fitting."]
                 raise RuntimeError("\n".join(msg))
             cv = (
-                np.dot(parameters, [v ** 4, v ** 3, v ** 2, v, 1])
+                np.dot(parameters, [v**4, v**3, v**2, v, 1])
                 / v
                 / 1000
                 / EvTokJmol
