@@ -1,11 +1,12 @@
 """Tests for irreducible representation calculations."""
 import os
 from io import StringIO
+from typing import List
 
 import numpy as np
 
+import phonopy
 from phonopy import Phonopy
-from phonopy.file_IO import parse_FORCE_SETS
 from phonopy.interface.vasp import read_vasp
 
 data_dir = os.path.dirname(os.path.abspath(__file__))
@@ -669,8 +670,10 @@ def test_pt03_P2():
     data = _load_data(chars_P2)
     phonon = _get_phonon("P2", [3, 2, 2], np.eye(3))
     phonon.set_irreps([0, 0, 0])
-    chars = phonon.get_irreps().get_characters()
+    chars = phonon.get_irreps().characters
     np.testing.assert_allclose(chars, data, atol=1e-5)
+    qpoints = [[0, 0, 0.5]]  # V
+    _check_char_sum(phonon, qpoints)
 
 
 def test_pt04_Pc():
@@ -678,8 +681,10 @@ def test_pt04_Pc():
     data = _load_data(chars_Pc)
     phonon = _get_phonon("Pc", [2, 2, 2], np.eye(3))
     phonon.set_irreps([0, 0, 0])
-    chars = phonon.get_irreps().get_characters()
+    chars = phonon.get_irreps().characters
     np.testing.assert_allclose(chars, data, atol=1e-5)
+    qpoints = [[0, 0.5, 0]]  # GA
+    _check_char_sum(phonon, qpoints)
 
 
 def test_pt06_P222_1():
@@ -687,8 +692,18 @@ def test_pt06_P222_1():
     data = _load_data(chars_P222_1)
     phonon = _get_phonon("P222_1", [2, 2, 1], np.eye(3))
     phonon.set_irreps([0, 0, 0])
-    chars = phonon.get_irreps().get_characters()
+    chars = phonon.get_irreps().characters
     np.testing.assert_allclose(chars, data, atol=1e-5)
+    qpoints = [
+        [0.5, 0, 0],
+        [0, 0.5, 0],
+        [0, 0, 0.5],
+        [0.5, 0.5, 0],
+        [0.5, 0, 0.5],
+        [0, 0.5, 0.5],
+        [0.5, 0.5, 0.5],
+    ]  # X, Y, Z, S, U, T, R
+    _check_char_sum(phonon, qpoints)
 
 
 def test_pt07_Amm2():
@@ -696,8 +711,16 @@ def test_pt07_Amm2():
     data = _load_data(chars_Amm2)
     phonon = _get_phonon("Amm2", [3, 2, 2], [[1, 0, 0], [0, 0.5, -0.5], [0, 0.5, 0.5]])
     phonon.set_irreps([0, 0, 0])
-    chars = phonon.get_irreps().get_characters()
+    chars = phonon.get_irreps().characters
     np.testing.assert_allclose(chars, data, atol=1e-5)
+    qpoints = [
+        [0, 0, -0.5],
+        [0.5, 0, -0.5],
+        [0.5, 0, 0],
+        [0, 0.25, -0.25],
+        [0.5, 0.25, -0.25],
+    ]
+    _check_char_sum(phonon, qpoints)
 
 
 def test_pt09_P4_1():
@@ -705,8 +728,10 @@ def test_pt09_P4_1():
     data = _load_data(chars_P4_1)
     phonon = _get_phonon("P4_1", [2, 2, 1], np.eye(3))
     phonon.set_irreps([0, 0, 0])
-    chars = phonon.get_irreps().get_characters()
+    chars = phonon.get_irreps().characters
     np.testing.assert_allclose(chars, data, atol=1e-5)
+    qpoints = [[0, 0, 0.5]]
+    _check_char_sum(phonon, qpoints)
 
 
 def test_pt10_Pbar4():
@@ -714,8 +739,14 @@ def test_pt10_Pbar4():
     data = _load_data(chars_Pbar4)
     phonon = _get_phonon("P-4", [1, 1, 2], np.eye(3))
     phonon.set_irreps([0, 0, 0])
-    chars = phonon.get_irreps().get_characters()
+    chars = phonon.get_irreps().characters
     np.testing.assert_allclose(chars, data, atol=1e-5)
+    qpoints = [
+        [0, 0, 0.5],
+        [0.5, 0.5, 0],
+        [0.5, 0.5, 0.5],
+    ]  # Z, M, A
+    _check_char_sum(phonon, qpoints)
 
 
 def test_pt11_I4_1a():
@@ -725,8 +756,15 @@ def test_pt11_I4_1a():
         "I4_1a", [2, 2, 1], np.array([[-1, 1, 1], [1, -1, 1], [1, 1, -1]]) * 0.5
     )
     phonon.set_irreps([0, 0, 0])
-    chars = phonon.get_irreps().get_characters()
+    chars = phonon.get_irreps().characters
     np.testing.assert_allclose(chars, data, atol=1e-5)
+    qpoints = [
+        [0.5, 0.5, -0.5],
+        [0, 0, 0.5],
+        [0.25, 0.25, 0.25],
+        [0, 0.5, 0],
+    ]  # M, X, P, N
+    _check_char_sum(phonon, qpoints)
 
 
 def test_pt13_P4mm():
@@ -734,8 +772,14 @@ def test_pt13_P4mm():
     data = _load_data(chars_P4mm)
     phonon = _get_phonon("P4mm", [3, 3, 2], np.eye(3))
     phonon.set_irreps([0, 0, 0])
-    chars = phonon.get_irreps().get_characters()
+    chars = phonon.get_irreps().characters
     np.testing.assert_allclose(chars, data, atol=1e-5)
+    qpoints = [
+        [0, 0, 0.5],
+        [0.5, 0.5, 0.5],
+        [0, 0.5, 0.5],
+    ]  # LD, V, W
+    _check_char_sum(phonon, qpoints)
 
 
 def test_pt14_Pbar42_1m():
@@ -743,8 +787,16 @@ def test_pt14_Pbar42_1m():
     data = _load_data(chars_Pbar42_1m)
     phonon = _get_phonon("P-42_1m", [2, 2, 3], np.eye(3))
     phonon.set_irreps([0, 0, 0])
-    chars = phonon.get_irreps().get_characters()
+    chars = phonon.get_irreps().characters
     np.testing.assert_allclose(chars, data, atol=1e-5)
+    qpoints = [
+        [0.5, 0.5, 0.5],
+        [0, 0, 0.5],
+        [0.5, 0.5, 0],
+        [0, 0.5, 0],
+        [0, 0.5, 0.5],
+    ]  # A, Z, M, X, R
+    _check_char_sum(phonon, qpoints)
 
 
 def test_pt19_P3m1():
@@ -752,8 +804,13 @@ def test_pt19_P3m1():
     data = _load_data(chars_P3m1)
     phonon = _get_phonon("P3m1", [4, 4, 2], np.eye(3))
     phonon.set_irreps([0, 0, 0])
-    chars = phonon.get_irreps().get_characters()
+    chars = phonon.get_irreps().characters
     np.testing.assert_allclose(chars, data, atol=1e-5)
+    qpoints = [
+        [0, 0, 0.5],
+        [1.0 / 3, 1.0 / 3, 1 / 2],
+    ]  # DT, P
+    _check_char_sum(phonon, qpoints)
 
 
 def test_pt20_Pbar3m1():
@@ -761,8 +818,16 @@ def test_pt20_Pbar3m1():
     data = _load_data(chars_Pbar3m1)
     phonon = _get_phonon("P-3m1", [3, 3, 2], np.eye(3))
     phonon.set_irreps([0, 0, 0])
-    chars = phonon.get_irreps().get_characters()
+    chars = phonon.get_irreps().characters
     np.testing.assert_allclose(chars, data, atol=1e-5)
+    qpoints = [
+        [0, 0, 0.5],
+        [1.0 / 3, 1.0 / 3, 0],
+        [1.0 / 3, 1.0 / 3, 0.5],
+        [0.5, 0, 0],
+        [0.5, 0, 0.5],
+    ]  # A, K, H, M, L
+    _check_char_sum(phonon, qpoints)
 
 
 def test_pt21_P6():
@@ -770,8 +835,13 @@ def test_pt21_P6():
     data = _load_data(chars_P6)
     phonon = _get_phonon("P6", [2, 2, 1], np.eye(3))
     phonon.set_irreps([0, 0, 0])
-    chars = phonon.get_irreps().get_characters()
+    chars = phonon.get_irreps().characters
     np.testing.assert_allclose(chars, data, atol=1e-5)
+    qpoints = [
+        [0, 0, 0.5],
+        [1.0 / 3, 1.0 / 3, 0.5],
+    ]  # DT, H
+    _check_char_sum(phonon, qpoints)
 
 
 def test_pt22_Pbar6():
@@ -779,8 +849,16 @@ def test_pt22_Pbar6():
     data = _load_data(chars_Pbar6)
     phonon = _get_phonon("P-6", [1, 1, 3], np.eye(3))
     phonon.set_irreps([0, 0, 0])
-    chars = phonon.get_irreps().get_characters()
+    chars = phonon.get_irreps().characters
     np.testing.assert_allclose(chars, data, atol=1e-5)
+    qpoints = [
+        [0, 0, 0.5],
+        [2.0 / 3, -1.0 / 3, 0],
+        [2.0 / 3, -1.0 / 3, 0.5],
+        [1.0 / 3, 1.0 / 3, 0],
+        [1.0 / 3, 1.0 / 3, 0.5],
+    ]  # A, KA, HA, K, H
+    _check_char_sum(phonon, qpoints)
 
 
 def test_pt24_P6_222():
@@ -788,8 +866,16 @@ def test_pt24_P6_222():
     data = _load_data(chars_P6_222)
     phonon = _get_phonon("P6_222", [2, 2, 2], np.eye(3))
     phonon.set_irreps([0, 0, 0])
-    chars = phonon.get_irreps().get_characters()
+    chars = phonon.get_irreps().characters
     np.testing.assert_allclose(chars, data, atol=1e-5)
+    qpoints = [
+        [0, 0, 0.5],
+        [1.0 / 3, 1.0 / 3, 0],
+        [1.0 / 3, 1.0 / 3, 0.5],
+        [0.5, 0, 0],
+        [0.5, 0, 0.5],
+    ]  # A, K, H, M, L
+    _check_char_sum(phonon, qpoints)
 
 
 def test_pt26_Pbar6m2():
@@ -797,8 +883,14 @@ def test_pt26_Pbar6m2():
     data = _load_data(chars_Pbar6m2)
     phonon = _get_phonon("P-6m2", [2, 2, 3], np.eye(3))
     phonon.set_irreps([0, 0, 0])
-    chars = phonon.get_irreps().get_characters()
+    chars = phonon.get_irreps().characters
     np.testing.assert_allclose(chars, data, atol=1e-5)
+    qpoints = [
+        [0, 0, 0.5],
+        [1.0 / 3, 1.0 / 3, 0],
+        [1.0 / 3, 1.0 / 3, 0.5],
+    ]  # A, K, H
+    _check_char_sum(phonon, qpoints)
 
 
 def test_pt28_P2_13():
@@ -806,8 +898,14 @@ def test_pt28_P2_13():
     data = _load_data(chars_P2_13)
     phonon = _get_phonon("P2_13", [2, 2, 2], np.eye(3))
     phonon.set_irreps([0, 0, 0])
-    chars = phonon.get_irreps().get_characters()
+    chars = phonon.get_irreps().characters
     np.testing.assert_allclose(chars, data, atol=1e-5)
+    qpoints = [
+        [0.5, 0.5, 0.5],
+        [0.5, 0.5, 0],
+        [0, 0.5, 0],
+    ]  # R, M, X
+    _check_char_sum(phonon, qpoints)
 
 
 def test_pt29_Pabar3():
@@ -815,8 +913,14 @@ def test_pt29_Pabar3():
     data = _load_data(chars_Pabar3)
     phonon = _get_phonon("Pa-3", [2, 2, 2], np.eye(3))
     phonon.set_irreps([0, 0, 0])
-    chars = phonon.get_irreps().get_characters()
+    chars = phonon.get_irreps().characters
     np.testing.assert_allclose(chars, data, atol=1e-5)
+    qpoints = [
+        [0.5, 0.5, 0.5],
+        [0.5, 0.5, 0],
+        [0, 0.5, 0],
+    ]  # R, M, X
+    _check_char_sum(phonon, qpoints)
 
 
 def test_pt30_P4_332():
@@ -824,8 +928,14 @@ def test_pt30_P4_332():
     data = _load_data(chars_P4_332)
     phonon = _get_phonon("P4_332", [1, 1, 1], np.eye(3))
     phonon.set_irreps([0, 0, 0])
-    chars = phonon.get_irreps().get_characters()
+    chars = phonon.get_irreps().characters
     np.testing.assert_allclose(chars, data, atol=1e-5)
+    qpoints = [
+        [0.5, 0.5, 0.5],
+        [0.5, 0.5, 0],
+        [0, 0.5, 0],
+    ]  # R, M, X
+    _check_char_sum(phonon, qpoints)
 
 
 def test_pt31_Pbar43m():
@@ -833,17 +943,26 @@ def test_pt31_Pbar43m():
     data = _load_data(chars_Pbar43m)
     phonon = _get_phonon("P-43m", [2, 2, 2], np.eye(3))
     phonon.set_irreps([0, 0, 0])
-    chars = phonon.get_irreps().get_characters()
+    chars = phonon.get_irreps().characters
     np.testing.assert_allclose(chars, data, atol=1e-5)
+    qpoints = [
+        [0.5, 0.5, 0.5],
+        [0.5, 0.5, 0],
+        [0, 0.5, 0],
+    ]  # R, M, X
+    _check_char_sum(phonon, qpoints)
 
 
 def _get_phonon(spgtype, dim, pmat):
     cell = read_vasp(os.path.join(data_dir, "POSCAR_%s" % spgtype))
-    phonon = Phonopy(cell, np.diag(dim), primitive_matrix=pmat)
     filename = os.path.join(data_dir, "FORCE_SETS_%s" % spgtype)
-    force_sets = parse_FORCE_SETS(filename=filename)
-    phonon.dataset = force_sets
-    phonon.produce_force_constants()
+    phonon = phonopy.load(
+        unitcell=cell,
+        supercell_matrix=np.diag(dim),
+        primitive_matrix=pmat,
+        force_sets_filename=filename,
+        symmetrize_fc=False,
+    )
     print(phonon.symmetry.pointgroup_symbol)
     return phonon
 
@@ -858,3 +977,36 @@ def _load_data(data_str):
     data = np.loadtxt(StringIO(data_str))
     data = data.view(dtype="c%d" % (data.itemsize * 2))
     return data
+
+
+def _check_char_sum(phonon: Phonopy, qpoints: List):
+    print("space-group number:", phonon.symmetry.dataset["number"])
+
+    for q in qpoints:
+        phonon.set_irreps(q)
+        print(phonon.get_irreps().qpoint, end="")
+        order = len(phonon.get_irreps().conventional_rotations)
+        char_sums = []
+        for i, (irreps, chars) in enumerate(
+            zip(phonon.get_irreps().irreps, phonon.get_irreps().characters)
+        ):
+            char_sum = 0
+            for irrep, char in zip(irreps, chars):
+                _char = np.trace(irrep)
+                np.testing.assert_almost_equal(_char, char)
+                char_sum += char.conj() * char
+            char_sum = abs(char_sum)
+            dim = int(np.rint(char_sum / order))
+            assert abs(char_sum - dim * order) < 1e-5
+            # print(i + 1, char_sum, order)
+            assert dim in [1, 2, 4]
+            if dim == 1:  # maybe physically irreducible (dim > 1)
+                char_sums.append(char_sum)
+
+        if char_sums:  # Can be empty due to physically irreducible irreps.
+            np.testing.assert_allclose(char_sums, order, atol=1e-5)
+
+        if len(char_sums) < len(phonon.get_irreps().irreps):
+            print("  ** Physically irreps may exist. **")
+        else:
+            print()
